-- DROP PROCEDURE IF EXISTS [dbo].[PROC_ADD_PERMISSAO_INTERFACE]

/*
 * FUNCIONALIDADE: Insere permissões de acesso a interfaces a partir dos módulos cadastrados
 * 1 - Busca todos as interfaces cadastrados no ICoruja
 * 2 - Une as interfaces com os id dos módulos onde a interface está configurada
 * 3 - Une com o id do perfis de usuário que possuem permissão de acesso ao módulo
 * 4 - Verificar se há opção de menu com permissão para o interface/modulo/perfil
 * 5 - Caso não haja, verifica se há opção de menu com acesso negado
 * 5.1 - Se sim, altera permissão de acesso a opção de menu para o perfil
 * 5.2 - Se não, verifica se existe opção de menu cadastrada
 * 5.2.1 - Se sim, cria permissão a opção de menu
 * 5.2.2 - Se não, cria opção de menu
 * 5.2.2.1 - Cria permissão a opção de menu
 * DATE: 15/05/2023
 * AUTOR: andre.zappe
 * PARAMETROS: Sem parâmetros
 * EX: EXEC PROC_ADD_PERMISSAO_INTERFACE;
*/

CREATE PROCEDURE [dbo].[PROC_ADD_PERMISSAO_INTERFACE] AS
BEGIN

    DECLARE @INTERFACE_CODTEL VARCHAR(255)
    DECLARE @INTERFACE_NOME VARCHAR(255)
    DECLARE @MODULO_ID INT
    DECLARE @PERFIL_ID INT
    DECLARE @COUNT_PERMISSAO_INTERFACE INT
    DECLARE @PERMISSAO_MENU_NEGADO_ID INT
    DECLARE @OPCAO_MENU_ID INT

    print '[START] PROC_ADD_PERMISSAO_INTERFACE ';

    DECLARE CURSOR_INTERFACES CURSOR FAST_FORWARD FOR -- BUSCA INTERFACES, MÓDULOS E PERFIS COM ACESSO AO MÓDULO
        SELECT DISTINCT FRM_CODFRM, FRM_TITULO, MOD_ID, PFM_PUSID FROM (
           SELECT FRM_CODFRM, FRM_TITULO, MOD_ID FROM GINTB_FORM FRM
                                                          LEFT JOIN GINTB_GRUPO_FORM GFR ON (FRM.FRM_GFRCODIGO = GFR.GFR_CODIGO)
                                                          INNER JOIN EDUSOFT_SECURITY.DBO.TB_MODULO ON (FRM.FRM_MOD_DISCP like '%' + MOD_TIPOMOD + '%')
           WHERE FRM_TIPO IN ('DOCUM_TEMPLATE_HTML', 'DOCUM_TEMPLATE_ODT', 'CRUD')
             AND FRM_EXECSOMPROC = 1
       ) TABELA, EDUSOFT_SECURITY.DBO.TB_PERFIL_MODULO
        WHERE PFM_MODID = MOD_ID AND PFM_PUSID IS NOT NULL
    OPEN CURSOR_INTERFACES
    FETCH NEXT FROM CURSOR_INTERFACES INTO @INTERFACE_CODTEL, @INTERFACE_NOME, @MODULO_ID, @PERFIL_ID
    WHILE @@FETCH_STATUS = 0
        BEGIN

            print '>>>>@INTERFACE_CODTEL - ' + @INTERFACE_CODTEL + ' | @INTERFACE_NOME - ' + @INTERFACE_NOME + ' | @MODULO_ID - ' + CAST(@MODULO_ID AS VARCHAR) + ' | @PERFIL_ID - ' + CAST(@PERFIL_ID AS VARCHAR)

            -- BUSCA SE HÁ ALGUMA PERMISSÃO PARA O INTERFACE, NO MÓDULO PARA O PERFIL DE USUÁRIO
            SET @COUNT_PERMISSAO_INTERFACE = (SELECT COUNT(*) FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                                   INNER JOIN EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO ON (OME_ID = OMP_OMEID AND OMP_PUSID = @PERFIL_ID)
                                              WHERE (OME_PREDEFID = @INTERFACE_CODTEL) AND
                                                      OME_TIPO = 'OPCAONOR' AND
                                                      OME_TIPOITEMPREDED = 'DYNAFORM' AND
                                                      OME_MODID = @MODULO_ID AND
                                                      OMP_TIPPER != 'TPA1')

            print '-----@COUNT_PERMISSAO_INTERFACE - ' + CAST(@COUNT_PERMISSAO_INTERFACE AS VARCHAR)

            IF @COUNT_PERMISSAO_INTERFACE = 0 -- CASO NÃO HAJA PERMISSÃO
                BEGIN
                    --                 -- BUSCA SE HÁ ALGUMA PERMISSÃO NEGANDO O ACESSO AO MENU
                    SET @PERMISSAO_MENU_NEGADO_ID = (SELECT TOP 1 OMP_ID FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                                                                  INNER JOIN EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO ON (OME_ID = OMP_OMEID AND OMP_PUSID = @PERFIL_ID)
                                                     WHERE (OME_PREDEFID = @INTERFACE_CODTEL) AND
                                                             OME_TIPO = 'OPCAONOR' AND
                                                             OME_TIPOITEMPREDED = 'DYNAFORM' AND
                                                             OME_MODID = @MODULO_ID AND
                                                             OMP_TIPPER = 'TPA1')

                    print '-----@PERMISSAO_MENU_NEGADO_ID - ' + CAST(COALESCE(@PERMISSAO_MENU_NEGADO_ID, 0) AS VARCHAR)

                    IF @PERMISSAO_MENU_NEGADO_ID IS NOT NULL -- HÁ PERMISSÃO CADASTRADA CONTUDO ESTÁ NEGADA, ALTERARÁ PARA POSSUIR PERMISSÃO NA OPÇÃO DE MENU
                        BEGIN
                            print '-----UPDATE TB_OPCAO_MENU_PERMISSAO'

                            UPDATE EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO SET OMP_TIPPER = 'TPA3', DATA_ULT_ALTERACAO = current_timestamp, USUARIO_ULT_ALTERACAO = 'EDUSOFT_PERMISSAO_INTERFACES', VERSAO = VERSAO + 1 WHERE OMP_ID = @PERMISSAO_MENU_NEGADO_ID
                        END
                    ELSE -- SE NÃO HÁ PERMISSÃO CADASTRADA, VERIFICA/CRIA A OPÇÃO DE MENU PARA O INTERFACE E CRIA A PERMISSÃO PARA O PERFIL DE USUÁRIO
                        BEGIN
                            -- VERIFICA SE HÁ OPÇÃO DE MENU CRIADA
                            SET @OPCAO_MENU_ID = (SELECT TOP 1 OME_ID FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                                  WHERE (OME_PREDEFID = @INTERFACE_CODTEL) AND
                                                          OME_TIPO = 'OPCAONOR' AND
                                                          OME_TIPOITEMPREDED = 'DYNAFORM' AND
                                                          OME_MODID = @MODULO_ID)

                            print '-----@OPCAO_MENU_ID - ' + CAST(COALESCE(@OPCAO_MENU_ID, 0) AS VARCHAR)

                            IF @OPCAO_MENU_ID IS NULL -- CRIA A OPÇÃO DE MENU PARA O INTERFACE
                                BEGIN
                                    print '-----INSERT TB_OPCAO_MENU'

                                    INSERT INTO EDUSOFT_SECURITY.dbo.TB_OPCAO_MENU (OME_DESCRICAO, OME_ACTION, OME_MODID, OME_ORDEM, OME_TIPO, VERSAO, DATA_ULT_ALTERACAO, USUARIO_ULT_ALTERACAO, OME_TIPACEITAPER, OME_TIPOMENU, OME_TIPOACAOMENU, OME_TIPOITEMPREDED, OME_PREDEFID, OME_CODRELATORIO, OME_PARAMETROS, OME_EXIMOB)
                                    VALUES (@INTERFACE_NOME, 'jsf/geradorinterface/executa/interfaceexec.jsf?codigoForm=' + @INTERFACE_CODTEL, @MODULO_ID, 0, 'OPCAONOR', 0, current_timestamp, 'EDUSOFT_PERMISSAO_INTERFACES', 'TAP3', 'ITEMMENU', 'PREDEF', 'DYNAFORM', @INTERFACE_CODTEL, '', '', 0);

                                    SET @OPCAO_MENU_ID = (SELECT TOP 1 OME_ID FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                                          WHERE (OME_PREDEFID = @INTERFACE_CODTEL) AND
                                                                  OME_TIPO = 'OPCAONOR' AND
                                                                  OME_TIPOITEMPREDED = 'DYNAFORM' AND
                                                                  OME_MODID = @MODULO_ID)

                                    print '-----@OPCAO_MENU_ID - ' + CAST(COALESCE(@OPCAO_MENU_ID, 0) AS VARCHAR)
                                END

                            print '-----INSERT TB_OPCAO_MENU_PERMISSAO'

                            -- CRIA PERMISSÃO PARA O MENU
                            INSERT INTO EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO (OMP_PUSID, OMP_OMEID, OMP_TIPPER, VERSAO, DATA_ULT_ALTERACAO, USUARIO_ULT_ALTERACAO)
                            VALUES (@PERFIL_ID, @OPCAO_MENU_ID, 'TPA3', 0, current_timestamp, 'EDUSOFT_PERMISSAO_INTERFACES')
                        END
                END

            FETCH NEXT FROM CURSOR_INTERFACES INTO @INTERFACE_CODTEL, @INTERFACE_NOME, @MODULO_ID, @PERFIL_ID
        END
    CLOSE CURSOR_INTERFACES
    DEALLOCATE CURSOR_INTERFACES

    print '[END] PROC_ADD_PERMISSAO_INTERFACE '
END