-- DROP PROCEDURE IF EXISTS [dbo].[PROC_ADD_PERMISSAO_RELATORIOS]

/* 
 * FUNCIONALIDADE: Insere permissões de acesso a relatórios a partir dos grupos de relatórios configurados no ICoruja 
 * 1 - Busta todos os grupos de relatórios cadastrados no ICoruja
 * 2 - Com os grupos de relatórios, busca todos os relatórios pertencentes a esses grupos
 * 3 - Une os relatórios com os id dos módulos onde dos grupos de relatórios estão configurados
 * 4 - Une com o id do perfis de usuário que possuem permissão de acesso ao módulo
 * 5 - Verificar se há opção de menu com permissão para o relatório/modulo/perfil
 * 6 - Caso não haja, verifica se há opção de menu com acesso negado
 * 6.1 - Se sim, altera permissão de acesso a opção de menu para o perfil
 * 6.2 - Se não, verifica se existe opção de menu cadastrada
 * 6.2.1 - Se sim, cria permissão a opção de menu
 * 6.2.2 - Se não, cria opção de menu
 * 6.2.2.1 - Cria permissão a opção de menu 
 * DATE: 27/04/2023
 * AUTOR: andre.zappe
 * PARAMETROS: Sem parâmetros
 * EX: EXEC PROC_ADD_PERMISSAO_RELATORIOS;
*/ 

CREATE PROCEDURE [dbo].[PROC_ADD_PERMISSAO_RELATORIOS] AS
BEGIN

    DECLARE @RELATORIO_CODTEL VARCHAR(255)
    DECLARE @RELATORIO_NOME VARCHAR(255)
    DECLARE @MODULO_ID INT
    DECLARE @PERFIL_ID INT
    DECLARE @COUNT_PERMISSAO_RELATORIO INT
    DECLARE @PERMISSAO_MENU_NEGADO_ID INT
    DECLARE @OPCAO_MENU_ID INT

    print '[START] PROC_ADD_PERMISSAO_RELATORIOS ';

    DECLARE CURSOR_RELATORIOS CURSOR FAST_FORWARD FOR -- BUSCA RELATÓRIOS, MÓDULOS E PERFIS COM ACESSO AO MÓDULO
        SELECT DISTINCT REL_CODTEL, REL_NOME, MOD_ID, PFM_PUSID FROM (
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT RGM_GRURELREMAN AS GRUPO, MOD_ID FROM TB_REGRA_MATRICULA, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE RGM_GRURELREMAN IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSU_GRURELCER AS GRUPO, MOD_ID FROM TB_PARAMETROS_SUBNIVEL_UNID, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE PSU_GRURELCER IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSU_GRURELMLO AS GRUPO, MOD_ID FROM TB_PARAMETROS_SUBNIVEL_UNID, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE PSU_GRURELMLO IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSU_GRURELHIS AS GRUPO, MOD_ID FROM TB_PARAMETROS_SUBNIVEL_UNID, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE PSU_GRURELHIS IS NOT NULL AND MOD_TIPOMOD = 'ACADEMICO') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELDOS AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELDOS IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT CPA_GRURELCTBL AS GRUPO, MOD_ID FROM TB_CONT_PARAMETROS, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE CPA_GRURELCTBL IS NOT NULL AND MOD_TIPOMOD = 'CONTABIL') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PFI_GRURELDIVATIID AS GRUPO, MOD_ID FROM TB_PARAMETROS_FINANCEIRO, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE PFI_GRURELDIVATIID IS NOT NULL AND MOD_TIPOMOD = 'FINANCEIRO') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELCPG AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELCPG IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELREN AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELREN IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELCAI AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELCAI IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELCPGCARTAO AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELCPGCARTAO IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELAVA AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELAVA IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PSE_GRURELINS AS GRUPO, PSE_MODID AS MOD_ID FROM TB_PARAMETROS_MODULO WHERE PSE_GRURELINS IS NOT NULL) AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
             UNION
             SELECT REL_CODTEL, REL_NOME, MOD_ID FROM EDUSOFT_REPORT.DBO.TB_REL_RELATORIO, (SELECT DISTINCT PRE_GRUREL AS GRUPO, MOD_ID FROM TB_PARAMETROS_REQUERIMENTO, EDUSOFT_SECURITY.DBO.TB_MODULO WHERE PRE_GRUREL IS NOT NULL AND MOD_TIPOMOD = 'REQUERIMEN') AS TABELA WHERE REL_GRUID IN (TABELA.GRUPO)
         ) TABELA, EDUSOFT_SECURITY.DBO.TB_PERFIL_MODULO
        WHERE PFM_MODID = MOD_ID AND PFM_PUSID IS NOT NULL
    OPEN CURSOR_RELATORIOS
    FETCH NEXT FROM CURSOR_RELATORIOS INTO @RELATORIO_CODTEL, @RELATORIO_NOME, @MODULO_ID, @PERFIL_ID
    WHILE @@FETCH_STATUS = 0
        BEGIN

        print '>>>>@RELATORIO_CODTEL - ' + @RELATORIO_CODTEL + ' | @RELATORIO_NOME - ' + @RELATORIO_NOME + ' | @MODULO_ID - ' + CAST(@MODULO_ID AS VARCHAR) + ' | @PERFIL_ID - ' + CAST(@PERFIL_ID AS VARCHAR)

        -- BUSCA SE HÁ ALGUMA PERMISSÃO PARA O RELATÓRIO, NO MÓDULO PARA O PERFIL DE USUÁRIO
        SET @COUNT_PERMISSAO_RELATORIO = (SELECT COUNT(*) FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                 INNER JOIN EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO ON (OME_ID = OMP_OMEID AND OMP_PUSID = @PERFIL_ID)
                                 WHERE (OME_ACTION = @RELATORIO_CODTEL OR OME_CODRELATORIO = @RELATORIO_CODTEL) AND
                                       OME_TIPO IN ('RELATORIO', 'OPCAONOR') AND
                                       OME_MODID = @MODULO_ID AND
                                       OMP_TIPPER != 'TPA1')

        print '-----@COUNT_PERMISSAO_RELATORIO - ' + CAST(@COUNT_PERMISSAO_RELATORIO AS VARCHAR)

        IF @COUNT_PERMISSAO_RELATORIO = 0 -- CASO NÃO HAJA PERMISSÃO
            BEGIN
--                 -- BUSCA SE HÁ ALGUMA PERMISSÃO NEGANDO O ACESSO AO MENU
                SET @PERMISSAO_MENU_NEGADO_ID = (SELECT TOP 1 OMP_ID FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                                        INNER JOIN EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO ON (OME_ID = OMP_OMEID AND OMP_PUSID = @PERFIL_ID)
                                                        WHERE (OME_ACTION = @RELATORIO_CODTEL OR OME_CODRELATORIO = @RELATORIO_CODTEL) AND
                                                          OME_TIPO = 'RELATORIO' AND
                                                          OME_MODID = @MODULO_ID AND
                                                          OMP_TIPPER = 'TPA1')

                print '-----@PERMISSAO_MENU_NEGADO_ID - ' + CAST(COALESCE(@PERMISSAO_MENU_NEGADO_ID, 0) AS VARCHAR)

                IF @PERMISSAO_MENU_NEGADO_ID IS NOT NULL -- HÁ PERMISSÃO CADASTRADA CONTUDO ESTÁ NEGADA, ALTERARÁ PARA POSSUIR PERMISSÃO NA OPÇÃO DE MENU
                    BEGIN
                        print '-----UPDATE TB_OPCAO_MENU_PERMISSAO'

                        UPDATE EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO SET OMP_TIPPER = 'TPA5', DATA_ULT_ALTERACAO = current_timestamp, USUARIO_ULT_ALTERACAO = 'EDUSOFT_PERMISSAO_RELATORIO', VERSAO = VERSAO + 1 WHERE OMP_ID = @PERMISSAO_MENU_NEGADO_ID
                    END
                ELSE -- SE NÃO HÁ PERMISSÃO CADASTRADA, VERIFICA/CRIA A OPÇÃO DE MENU PARA O RELATÓRIO E CRIA A PERMISSÃO PARA O PERFIL DE USUÁRIO
                    BEGIN
                        -- VERIFICA SE HÁ OPÇÃO DE MENU CRIADA
                        SET @OPCAO_MENU_ID = (SELECT TOP 1 OME_ID FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                              WHERE (OME_ACTION = @RELATORIO_CODTEL OR OME_CODRELATORIO = @RELATORIO_CODTEL) AND
                                                      OME_TIPO = 'RELATORIO' AND
                                                      OME_MODID = @MODULO_ID)

                        print '-----@OPCAO_MENU_ID - ' + CAST(COALESCE(@OPCAO_MENU_ID, 0) AS VARCHAR)

                        IF @OPCAO_MENU_ID IS NULL -- CRIA A OPÇÃO DE MENU PARA O RELATÓRIO
                            BEGIN
                                print '-----INSERT TB_OPCAO_MENU'

                                INSERT INTO EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU (OME_DESCRICAO, OME_ACTION, OME_MODID, OME_ORDEM, OME_TIPO, VERSAO, DATA_ULT_ALTERACAO, USUARIO_ULT_ALTERACAO, OME_TIPACEITAPER, OME_TIPOMENU, OME_TIPOACAOMENU, OME_CODRELATORIO, OME_PARAMETROS, OME_ICONE, OME_EXIMOB, OME_APPEXIBEPAGPRI)
                                VALUES (@RELATORIO_NOME, @RELATORIO_CODTEL, @MODULO_ID, 0, 'RELATORIO', 0, current_timestamp, 'EDUSOFT_PERMISSAO_RELATORIO', 'TAP1', 'RELATORIO', 'NORMAL', @RELATORIO_CODTEL, '', '', 0, 0)

                                SET @OPCAO_MENU_ID = (SELECT TOP 1 OME_ID FROM EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU
                                                      WHERE (OME_ACTION = @RELATORIO_CODTEL OR OME_CODRELATORIO = @RELATORIO_CODTEL) AND
                                                              OME_TIPO = 'RELATORIO' AND
                                                              OME_MODID = @MODULO_ID)

                                print '-----@OPCAO_MENU_ID - ' + CAST(COALESCE(@OPCAO_MENU_ID, 0) AS VARCHAR)
                            END

                        print '-----INSERT TB_OPCAO_MENU_PERMISSAO'

                        -- CRIA PERMISSÃO PARA O MENU
                        INSERT INTO EDUSOFT_SECURITY.DBO.TB_OPCAO_MENU_PERMISSAO (OMP_PUSID, OMP_OMEID, OMP_TIPPER, VERSAO, DATA_ULT_ALTERACAO, USUARIO_ULT_ALTERACAO)
                        VALUES (@PERFIL_ID, @OPCAO_MENU_ID, 'TPA5', 0, current_timestamp, 'EDUSOFT_PERMISSAO_RELATORIO')
                    END
            END

        FETCH NEXT FROM CURSOR_RELATORIOS INTO @RELATORIO_CODTEL, @RELATORIO_NOME, @MODULO_ID, @PERFIL_ID
        END
    CLOSE CURSOR_RELATORIOS
    DEALLOCATE CURSOR_RELATORIOS

    print '[END] PROC_ADD_PERMISSAO_RELATORIOS '
END
